/*******************************************************
 * File: AddressBookDialog.cpp
 * Authors: Jake Becker (jabecker@stanford.edu)
 *          Keith Schwarz (htiek@cs.stanford.edu)
 *
 * Implementation of the "Manage Address Book" window.  You
 * should not need to use or modify the contents of this file.
 */

#include <QtGui>
#include <QtDebug>
#include <vector>
#include "mail.h"
#include "AddressBookDialog.h"
#include "ui_addressbook.h"
using namespace std;

/** Module-Specific Functions **/

/* Returns whether something is a valid email address by simulating a DFA
 * that accepts email addresses on that input.  This is not well-commented
 * since you pretty much just have to look at the state transition diagram
 * to understand what you're looking at.
 */
static bool IsValidEmailAddress(QString str)
{
	/* Search for  a+ [. a+]* @ a* [. a+]+
	 *           0 1   2  3   4 5   6 7
	 */
	int state = 0;
	for (int i = 0; i < str.length(); ++i)
	{
		/* Can be alnum, ., or @. */
		if (str.at(i).isLetterOrNumber() || str.at(i) == '-')
		{
			switch(state)
			{
			case 0: state = 1; break;
			case 1: break;
			case 2: state = 3; break;
			case 3: break;
			case 4: state = 5; break;
			case 5: break;
			case 6: state = 7; break;
			case 7: break;
			}
		}
		else if (str.at(i) == '.')
		{
			switch(state)
			{
			case 0: return false;
			case 1: state = 2; break;
			case 2: return false;
			case 3: state = 2; break;
			case 4: return false;
			case 5: state = 6; break;
			case 6: return false;
			case 7: state = 6; break;
			}
		}
		else if (str.at(i) == '@')
		{
			switch(state)
			{
			case 0: return false;
			case 1: state = 4; break;
			case 2: return false;
			case 3: state = 4; break;
			case 4: return false;
			case 5: return false;
			case 6: return false;
			case 7: return false;
			}
		}
		else return false;
	}
	return state == 7;
}

/** Implementation of AddressBookDialog **/

/* Constructor wires up the buttons and stores which vector we should use
 * to store the result.
 */
AddressBookDialog::AddressBookDialog(const QStringList& addressList) : addressList(addressList)
{
	ui.setupUi( this );

	ui.addressesListView->setModel( &addressListModel );
	ui.addAddressButton->setDefault(true);

	connect( ui.addAddressButton, SIGNAL(clicked()), this, SLOT(addAddress()) );
	connect( ui.removeAddressButton, SIGNAL(clicked()), this, SLOT(removeAddress()) );
	connect( ui.buttonBox, SIGNAL(accepted()), this, SLOT(save()) );
}

/* Respond to a click of the "Add" button. */
void AddressBookDialog::addAddress()
{
        /* Grab the text out of the window. */
	QString address = ui.addressLineEdit->text();

	/* If we've already seen this, do nothing. */
	if(tempAddressList.contains(address))
	{
	        QMessageBox::warning(this, tr("CS106L SendMail"), tr("That address is already in the address book.") );
		ui.addressLineEdit->clear();
	}
	/* Otherwise, if this isn't a valid address, report that. */
 	else if (!IsValidEmailAddress(address))
	{
		QMessageBox::warning(this, tr("CS106L SendMail"), tr("Please enter a valid email address.") );
	}
	else
	{
		tempAddressList.append(address);
		addressListModel.setStringList(tempAddressList);
		ui.addressLineEdit->clear();
	}

}

/* Respond to a "remove" click. */
void AddressBookDialog::removeAddress()
{
	tempAddressList.removeAt( ui.addressesListView->currentIndex().row() );
	addressListModel.setStringList(tempAddressList);
}

/* Sets up the address book list and runs the dialog. */
int AddressBookDialog::exec()
{
	tempAddressList = addressList;
	addressListModel.setStringList(tempAddressList);
	return QDialog::exec();
}

/* Save changes to the address book. */
void AddressBookDialog::save()
{
        /* Convert the Qt list of strings into a vector of strings. */
	vector<string> addressBook;
	for(int i = 0; i < tempAddressList.count(); i++)
		addressBook.push_back(tempAddressList.at(i).toStdString());

	/* Call your function to save this list! */
	SaveAddressBook(addressBook);
	accept();
}
